"""
input.py

Handles input for the game.
"""

import sys

import pygame

from defend import HEIGHT, WIDTH

current_tool = None
current_cell = None

def get_advance():
  for e in pygame.event.get():
    if e.type == pygame.QUIT:
      sys.exit(0)
    elif e.type== pygame.KEYUP:
      return True
  return False

def get_input():
  global current_tool, current_cell
  commands = []
  for e in pygame.event.get():
    if e.type == pygame.QUIT:
      commands.append("quit")
    elif e.type == pygame.KEYUP:
      if e.key == pygame.K_ESCAPE:
        commands.append("quit")
      elif e.key == pygame.K_w:
        commands.append(("move", "north"))
      elif e.key == pygame.K_a:
        commands.append(("move", "west"))
      elif e.key == pygame.K_s:
        commands.append(("move", "south"))
      elif e.key == pygame.K_d:
        commands.append(("move", "east"))
    elif e.type == pygame.MOUSEMOTION:
      mx, my = e.pos
      if 0 <= my < HEIGHT:
        if 0 <= mx < WIDTH - 96:
          hovering = (mx // 32, my // 32)
          commands.append(("hover", hovering))
          commands.append(("hover_tool", None))
          current_cell = hovering
          current_tool = None
        elif WIDTH - 80 <= mx < WIDTH - 48:
          hovering = my // 32
          commands.append(("hover", None))
          if hovering % 2:
            hovering //= 2
            commands.append(("hover_tool", hovering))
            current_cell = None
            current_tool = hovering
          else:
            commands.append(("hover_tool", None))
            current_cell = None
            current_tool = None
        else:
          commands.append(("hover", None))
          commands.append(("hover_tool", None))
          current_cell = None
          current_tool = None
      else:
        commands.append(("hover", None))
        commands.append(("hover_tool", None))
        current_cell = None
        current_tool = None
    elif e.type == pygame.MOUSEBUTTONUP:
      if e.button == 1:
        if current_tool != None:
          commands.append(("select_tool", current_tool))
        elif current_cell:
          commands.append(("use_tool", current_cell))
  return commands
