"""
objects.py

Defines the various enemies and player units.
"""

import os, random

import pygame

SIZE = (21, 18)
HEART = (SIZE[0]//2, SIZE[1]//2)

def loadres(name):
  return pygame.image.load(os.path.join("res", name + ".png"))

class Thing:
  name = "<thing>"
  maxhealth = 1
  damage = 0
  loot_value = 1
  sprite = None

  def __init__(self, facing=None):
    self.health = self.maxhealth
    self.facing = facing

  def next_pos(self, state, x, y):
    if not self.facing:
      return None
    if self.facing == "north":
      next = (x, y - 1)
    elif self.facing == "east":
      next = (x + 1, y)
    elif self.facing == "south":
      next = (x, y + 1)
    elif self.facing == "west":
      next = (x - 1, y)
    return next

class Ally(Thing):
  name = "<ally>"

class Heart(Ally):
  name = "heart"
  maxhealth = 3
  sprite = loadres("heart")

class Guy(Ally):
  name ="guy"
  maxhealth = None
  sprite = loadres("guy")
  steal = 1
  capacity = 10
  spritefacing = None

  def next_pos(self, state, x, y):
    if self.facing in ("east", "west"):
      self.spritefacing = self.facing
    return super().next_pos(state, x, y)

class Wall(Ally):
  name = "wall"
  maxhealth = 2
  sprite = loadres("wall")
  cost = 3

class Barricade(Ally):
  name = "barricade"
  maxhealth = 3
  sprite = loadres("wall")
  cost = 3

class Turret(Ally):
  name = "<turret>"
  range = 3
  threshold = 1
  cost = 3
  attack_type = "red beam"
  loot_value = 2
  def __init__(self):
    super().__init__()
    self.charge = 0

class LightTurret(Turret):
  name = "light_turret"
  maxhealth = 4
  damage = 1
  range = 3
  threshold = 4
  cost = 12
  attack_type = "cucumber beam"
  sprite = loadres("light_turret")

class HeavyTurret(Turret):
  name = "heavy_turret"
  maxhealth = 5
  damage = 4
  range = 3
  threshold = 12
  cost = 14
  attack_type = "pea soup beam"
  sprite = loadres("heavy_turret")

class BurstTurret(Turret):
  name = "burst_turret"
  maxhealth = 4
  damage = 1
  range = 3
  threshold = 10
  cost = 15
  attack_type = "orange burst beam"
  sprite = loadres("burst_turret")

class TeslaTurret(Turret):
  name = "tesla_turret"
  maxhealth = 5
  damage = 1
  range = 1
  threshold = 2
  cost = 12
  attack_type = "tesla beam"
  sprite = loadres("tesla_turret")

class MagnetTurret(Turret):
  name = "magnet_turret"
  maxhealth = 3
  damage = 0
  range = 4
  threshold = 1
  cost = 10
  attack_type = "magnet beam"
  sprite = loadres("magnet_turret")

class Enemy(Thing):
  name = "<enemy>"
  damage = 1

  def __init__(self, facing=None):
    super().__init__(facing)
    self.moved = False

  def next_pos(self, state, x, y):
    eastwest = None
    if x < HEART[0] \
    and x < state.size[0] - 1 \
    and not isinstance(state.get(x + 1, y)["thing"], Enemy):
      eastwest = "east"
    elif x > HEART[0] \
    and x > 0 \
    and not isinstance(state.get(x - 1, y)["thing"], Enemy):
      eastwest = "west"
    northsouth = None 
    if y < HEART[1] \
    and y < state.size[1] - 1 \
    and not isinstance(state.get(x, y + 1)["thing"], Enemy):
      northsouth = "south"
    elif y > HEART[1] \
    and y > 0 \
    and not isinstance(state.get(x, y - 1)["thing"], Enemy):
      northsouth = "north"
    if eastwest:
      if northsouth:
        dir = random.choice((eastwest, northsouth))
      else:
        dir = eastwest
    else:
      dir = northsouth
    if not dir:
      dir = random.choice(("north", "south", "east", "west"))
    self.facing = dir
    if dir == "north":
      return (x, y - 1)
    elif dir == "south":
      return (x, y + 1)
    elif dir == "east":
      return (x + 1, y)
    elif dir == "west":
      return (x - 1, y)

class RamBot(Enemy):
  name = "rambot"
  maxhealth = 5
  damage = 1
  sprite = loadres("rambot")

class DrillBot(Enemy):
  name = "drillbot"
  maxhealth = 10
  damage = 2
  loot_value = 2
  sprite = loadres("drillbot")

class BombBot(Enemy):
  name = "bombbot"
  maxhealth = 10
  damage = 4
  loot_value = 3
  sprite = loadres("bombbot")
